package com.rhiannonweb.android.migrainetracker.data;

import java.text.SimpleDateFormat;
import java.util.Date;

import android.content.ContentValues;
import android.database.Cursor;
import android.provider.BaseColumns;
import android.util.Log;

import com.rhiannonweb.android.migrainetracker.R;

/**
 * Headache data structure to store per-headache information.
 * @author Chad Vincent
 *
 */
public class Headache {
	private static final long MINUTES_IN_MS = (60 * 1000);
	private static final long HOURS_IN_MS = (60 * MINUTES_IN_MS);
	private static final long DAYS_IN_MS = (24 * HOURS_IN_MS);
	
	private Date beginDate;
	private long dbID = 0;
	private Date endDate;
	private int severity;
	private String notes;
	
	/**
	 * Begin a new Headache now.
	 */
	public Headache() {
		beginDate = new Date(System.currentTimeMillis());
	}

	/**
	 * Begin a new Headache as of the specified date.
	 * @param begin Date depicting beginning of headache.
	 */
	public Headache(Date begin) {
		beginDate = begin;
	}

	/**
	 * Begin a new Headache as of the specified date.
	 * @param begin Date depicting beginning of headache.
	 */
	public Headache(long begin) {
		beginDate = new Date(begin);
	}

	public Headache(Cursor cursor) {
		dbID = cursor.getLong(cursor.getColumnIndex(BaseColumns._ID));
		SimpleDateFormat format = new SimpleDateFormat(MigraineOpen.DATE_FORMAT);
		try {
			beginDate = format.parse(cursor.getString(cursor.getColumnIndex(MigraineOpen.BEGIN_DATE)));
			severity = cursor.getInt(cursor.getColumnIndex(MigraineOpen.SEVERITY));
			endDate = format.parse(cursor.getString(cursor.getColumnIndex(MigraineOpen.END_DATE)));
			try {
				notes = cursor.getString(cursor.getColumnIndex(MigraineOpen.NOTES));
			} catch (NullPointerException bummer) {
				// Headache predates the Notes field, perhaps?
				notes = "";
			}
		} catch (Exception e) {
			Log.e(this.getClass().getCanonicalName(), "Error parsing dates out of DB", e);
		}
	}

	/**
	 * Set end Date of the Headache to right now.
	 */
	public void end() {
		endDate = new Date(System.currentTimeMillis());
	}

	/**
	 * Returns a String representation of the duration of the Headache.
	 * Format is [n Days] h:mm
	 * @return Representation of the duration in string format
	 */
	public String getDuration() {
		// TODO: Use regional/system formatting
		// TODO: This really should use Calendar instead of doing it manually.
		long diff;
		if (endDate != null) {
			diff = endDate.getTime() - beginDate.getTime();
		} else {
			diff = System.currentTimeMillis() - beginDate.getTime();
		}
		if (diff > DAYS_IN_MS) {
			return (diff / DAYS_IN_MS) + " " + R.string.days + " "
			+ ((diff%DAYS_IN_MS) / HOURS_IN_MS) + ":"
			+ String.format("%02d", (diff%HOURS_IN_MS) / MINUTES_IN_MS);
		} else {
			return ((diff) / HOURS_IN_MS) + ":"
			+ String.format("%02d", (diff%HOURS_IN_MS) / MINUTES_IN_MS);
		}
	}

	public int getSeverity() {
		return severity;
	}

	public void setSeverity(int severity) {
		this.severity = severity;
	}

	public Date getBeginDate() {
		return beginDate;
	}
	
	public void setBeginDate(Date beginDate) {
		this.beginDate = beginDate;
	}

	public Date getEndDate() {
		return endDate;
	}

	public void setEndDate(Date endDate) {
		this.endDate = endDate;
	}

	public long getDbID() {
		return dbID;
	}

	public void setDbID(long l) {
		this.dbID = l;
	}

	public String getNotes() {
		return notes;
	}

	public void setNotes(String notes) {
		this.notes = notes;
	}
	
	public ContentValues getSQLContentValues() {
		ContentValues ret = new ContentValues();
		
		ret.put(MigraineOpen.SEVERITY, severity);
		
		if (dbID > 0) {
			ret.put(BaseColumns._ID, dbID);
		}
		
		SimpleDateFormat format = new SimpleDateFormat(MigraineOpen.DATE_FORMAT);
		ret.put(MigraineOpen.BEGIN_DATE, format.format(beginDate));
		ret.put(MigraineOpen.END_DATE, format.format(endDate));
		ret.put(MigraineOpen.NOTES, notes);
		
		return ret;
	}

	@Override
	public String toString() {
		return "Headache " + getDbID() + " on " + getBeginDate() + " " + super.toString();
	}
}
